﻿using System;
using System.IO;
using Alphaleonis.Win32.Vss;

namespace Ariely_AlphaVSS_Sample01
{
    class Program
    {
        /// <summary>
        /// This is only first test of using AlphaVSS project. 
        /// Please make sure you read thoroughly and understand the code's comments and adapt it to your system and needs.
        /// We are using constant links for "source files to copy" and for the "destination folder".
        /// In any dynamic application you should get those values from the user.
        /// 
        /// building your project:
        /// -----------------------
        /// 1. Creating New Project
        ///    * Open Visual Studio As Administrator!
        ///    * Chose create new project type "Console application"
        ///    * use Dot.Net 4 framework
        ///    * named the project Ariely_AlphaVSS_Sample01
        /// 2. Add NuGet Package named alphavss (use the search on-line option, and install the package)
        /// 3. Go over the Code's comment and understand it!!! After you are ready you can use it.
        /// 4. add "using Alphaleonis.Win32.Vss;" & "using System.IO;"
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            String _Source1 = @"C:\_ISUG_VSS_DB\20140106_ISUG_VSS_DB.mdf";
            String _Source2 = @"C:\_ISUG_VSS_DB\20140106_ISUG_VSS_DB_log.ldf";
            String _Destination = @"V:\QQ";
            FileInfo MyFileInfo = new FileInfo(_Source1);
            String _Volume = MyFileInfo.Directory.Root.Name;

            IVssImplementation _vssImplementation = VssUtils.LoadImplementation();
            IVssBackupComponents _backup = _vssImplementation.CreateVssBackupComponents();
            _backup.InitializeForBackup(null);

            _backup.GatherWriterMetadata();

            _backup.SetContext(VssVolumeSnapshotAttributes.Persistent | VssVolumeSnapshotAttributes.NoAutoRelease);
            _backup.SetBackupState(false, true, Alphaleonis.Win32.Vss.VssBackupType.Full, false);

            Guid MyGuid01 = _backup.StartSnapshotSet();
            Guid MyGuid02 = _backup.AddToSnapshotSet(_Volume, Guid.Empty);

            _backup.PrepareForBackup();
            _backup.DoSnapshotSet();

            VssSnapshotProperties _MyProperties = _backup.GetSnapshotProperties(MyGuid02);

            // הפקודה הבאה תיצור לנו חשיפה של עותק הצל בעזרת שם של דיסק חדש מקומי או תיקייה שיתופית
            _backup.ExposeSnapshot(MyGuid02, null, VssVolumeSnapshotAttributes.ExposedLocally, "L:");

            // נעתיק את הקבצים עתה מהתמונה חשופה שלנו
            String sVSSFile1 = _Source1.Replace(_Volume, @"L:\");
            String sVSSFile2 = _Source2.Replace(_Volume, @"L:\");
            if (File.Exists(sVSSFile1))
                System.IO.File.Copy(sVSSFile1, _Destination + @"\" + System.IO.Path.GetFileName(_Source1), true);
            if (File.Exists(sVSSFile2))
                System.IO.File.Copy(sVSSFile2, _Destination + @"\" + System.IO.Path.GetFileName(_Source2), true);

            // Delete Exposed Snapshot
            foreach (VssSnapshotProperties prop in _backup.QuerySnapshots())
            {
                if (prop.ExposedName == @"L:\")
                {
                    Console.WriteLine("prop.ExposedNam Found!");
                    _backup.DeleteSnapshot(prop.SnapshotId, true);
                }
            }

            _backup = null;

            Console.WriteLine("END OK. Click any Key to close the application");
            Console.ReadLine();
        }
    }
}
